//
// Copyright 2002-2005 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Pdf.Kit for Java. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//
package samples.print;

import java.io.File;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import com.aspose.pdf.kit.PdfViewer;
import com.aspose.pdf.kit.PdfViewerException;

/**
 * <p>A simple example to background print Pdf files.
 * The SimpleViewer example shows all the PdfViewer lib functions.
 * </p>
 * <p>Copyright: 2002-2005 . All Rights Reserved.</p>
 * <p>Company: Aspose Pty Ltd</p>
 *
 * @version 1.0.0.0
 */

public class BackgroundPrint
{
	private static final long serialVersionUID = 100020050918L;
	
	/**
	 * the function point of the PdfViewer Lib.
	 */
	private PdfViewer pdfviewer;
	
	/**
	 * the path of current pdf file.
	 */
	private String currentPdfFilePath = null;

	/** 
	 * the current page number of the current Pdf file.
	 * The first page is 1.
	 */
	private int currentPage = 1;	
	
	/**
	 * page scale factor, 1 = 100%.
	 */
	protected float scaleFactor = 1;
	
	/** 
	 * the edge of page.
	 */
	private final int inset = 25;
	
	/** 
	 * the flag of print task.
	 */
	protected static int printingLocks = 0;
	
	static String password=" ";
	
	
	/**
	 * The constructor of the BackgroundPrint object.
	 * 
	 * @param name String the full file path.
	 * @param password String the password of the file.
	 */
	public BackgroundPrint(String filepath,String password)
	{

		pdfviewer = new PdfViewer();

		currentPdfFilePath = filepath;	

		try {
			if (pdfviewer.isEncryptedFile(currentPdfFilePath)) {
				if (pdfviewer.isOwnerPassword(currentPdfFilePath,password)|pdfviewer.isUserPassword(currentPdfFilePath,password)) {
					pdfviewer.openPdfFile(currentPdfFilePath, password);
				}
			} else {
				pdfviewer.openPdfFile(currentPdfFilePath);
			}
			pdfviewer.decodePage(currentPage);
		}

		catch (PdfViewerException ex) {
			System.out.println(ex);
		} catch (Exception ex) {
			System.out.println(ex);
		}		
	}

	/**
	 * Handles printing function using the Java print function to print pdf
	 * file.
	 */
	protected void printPdfFile()
	{	
		try{
			if (pdfviewer.isEncryptedFile(currentPdfFilePath)) {

				if (pdfviewer.isUserPassword(currentPdfFilePath, password)) {

					if (!pdfviewer.canUserPrint(currentPdfFilePath)) {
						System.out.print("You are a user not a owner and you haven't right to print.");
						password = null;
						return;
					}
				}
			}
		if (printingLocks != 0) { return; }

		//adds a print lock to ensure only a pdf file to print.
		printingLocks++;
		
		//prints in thread to improve background printing.		 
		Thread workerThread = new Thread() {
			public void run()
			{
				boolean canceled = false;
				try
				{
					//sets the default print scale - false.
					pdfviewer.enableScalePrint(false);

					//gets a printjob object.
					PrinterJob printJob = PrinterJob.getPrinterJob();
					
					//creates a new PageFormat instance and sets it to a default size and orientation.
					PageFormat pf = printJob.defaultPage();

					//sets PageOrientation to best use page layout.
					int orientation = pdfviewer.getPdfWidth() < pdfviewer.getPdfHeight() ? PageFormat.PORTRAIT : PageFormat.LANDSCAPE;
					pf.setOrientation(orientation);

					Paper paper = new Paper();
					
					// //A4  bordless
					//paper.setSize(595, 842);
					//paper.setImageableArea(0, 0, 595, 842);
					//          
					// //A5
					//paper.setSize(297, 421);
					//paper.setImageableArea(23,23,254,378);
		

					//A4 
					paper.setSize(595, 842);
					paper.setImageableArea(43, 43, 509, 756);                
				   
					//sets the Paper object for this PageFormat.
					pf.setPaper(paper);				
					
					//sets the start page for printing, and you can change it according to your request but make sure  1 =< startPage <= endPage <= pdfviewer.getPdfFilePageCount().
					int startPage = 1;
				  
				    //sets the end page for printing, and you can change it according to your request but make sure  1 =< startPage <= endPage <= pdfviewer.getPdfFilePageCount().
					int endPage = pdfviewer.getPdfFilePageCount();
					
					//sets the page range according to the user's decision.
					pdfviewer.setPrintPageRange(startPage,endPage);
					
					//sets the page format for each page between startPage and endPage.
					for(int page = startPage; page > endPage + 1; page++)
						pdfviewer.setPageFormat(page,pf); 
					  	
					//queries document for the number of pages and the PageFormat and Printable for each page held in the Pageable instance, document.
					printJob.setPageable(pdfviewer);
					
					//prints a set of pages.
					printJob.print();
				}
				catch (PrinterException ex)
				{
					System.out.println(ex);
				}
				catch (Exception ex)
				{
					System.out.println(ex);					
				}
				catch (Error err)
				{
					System.out.println(err);
				}
				
				//if problems, reports any errors.
				if (!canceled && !pdfviewer.isPrintSuccess())
					System.out.println("Problems encountered\n " + pdfviewer.getPrintFailureMessage() + "\n");				

				printingLocks--;

				if (!canceled)
				System.out.println("finished printing!");
			}
		};

		//starts printing in background.
		workerThread.start();
		}
		catch (Exception ex) {
			System.out.println(ex);
		}
	}

	/** 
	 * main function.
	 */
	public static void main(String[] args)
	{
		//if you got a license, uncomment following lines
//		License lic  = new License();
//		try 
//		{
//			lic.setLicense(new FileInputStream(new File("Aspose.Pdf.Kit.Java.Ent.lic")));
//		} catch (Exception e) {
//			System.out.println(e.getMessage());
//		}
    
		//the file path to be printed.
		String printPdfFilePath = "./examples/resources/printTest.pdf";
		
		password="testPassword";
		
		BackgroundPrint printDemo = null;
		
		//Check the file exist or not.
		File printFile = new File(printPdfFilePath);
		
		if (printFile.exists() == false)
			System.out.println("Error! -Has not Pdf file path to print.");
		else
		{
			//constructs an BackgroundPrint object.
			printDemo = new BackgroundPrint(printPdfFilePath,password);
			//calls the printPdfFile method to background print.
			printDemo.printPdfFile();
		}
	}	

}